<?php
header("Access-Control-Allow-Origin: *");
header("Access-Control-Expose-Headers: *");
//-----------------------------------------------------------------------------
// 地図情報システム Web-GIS
// Copyright(C) neoGIS  2001-2003
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// < Program Name >
//    SYSTEM       : 地図情報システム Web-GIS
//    PG-NAME      : sg_tenpu_save_ajax.php
//    AUTHOR       : sakurai
//    DATE-WRITTEN : 2022/12/06
// < Summary >
//-----------------------------------------------------------------------------
// < History >
//-----------------------------------------------------------------------------
ob_start();

function uniDecode($str, $charcode='UTF-8'){
	$text = preg_replace_callback(
		"/%u[0-9A-Za-z]{4}/", uniDecode_toUtf8, $str);
	return (($charcode != 'UTF-8')
		? mb_convert_encoding($text, $charcode, 'UTF-8') : $text);
}
function uniDecode_toUtf8($m){
	$c = '';
	foreach($m as $val){
		$val = intval(substr($val, 2), 16);
		if($val < 0x7F){        // 0000-007F
			$c .= chr($val);
		}elseif($val < 0x800) { // 0080-0800
			$c .= chr(0xC0 | ($val / 64));
			$c .= chr(0x80 | ($val % 64));
		}else{                // 0800-FFFF
			$c .= chr(0xE0 | (($val / 64) / 64));
			$c .= chr(0x80 | (($val / 64) % 64));
			$c .= chr(0x80 | ($val % 64));
		}
	}
	return $c;
}

function mb_utf8($str, $fromChar='UTF-8'){
	return mb_convert_encoding($str, 'UTF-8', $fromChar);
}

function mb_enc2locale($enc='UTF-8'){
	$lc = '';
	switch($enc){
	case 'SJIS': case 'SJIS-win': case 'CP932':
		$lc = 'ja_JP.SJIS'; break;
	case 'EUC-JP': case 'eucJP-win': case 'CP51932':
		$lc = 'ja_JP.eucJP'; break;
	case 'UTF-8': default:
		$lc = 'ja_JP.UTF8'; break;
	}
	return $lc;
}
function mb_basename($file, $sfx=NULL, $enc='UTF-8'){
	$lc = setlocale(LC_CTYPE, '0');
	setlocale(LC_CTYPE, mb_enc2locale($enc));
	$rc = (isset($sfx)) ? basename($file, $sfx) : basename($file);
	setlocale(LC_CTYPE, $lc);
	return $rc;
}
function mb_pathinfo($path, $opt=NULL, $enc='UTF-8'){
	$lc = setlocale(LC_CTYPE, '0');
	setlocale(LC_CTYPE, mb_enc2locale($enc));
	$rc = (isset($opt)) ? pathinfo($path, $opt) : pathinfo($path);
	setlocale(LC_CTYPE, $lc);
	return $rc;
}

define('_D_SJ5C_SW', (DIRECTORY_SEPARATOR !== '/')); // ※Windows環境のみ変換
function sj5c_isSJIS($tx, $sjEnc= 'SJIS-win'){
	// 文字コードがSJISであるかのチェック
	if(!_D_SJ5C_SW){ return FALSE; }
	$txUTF = mb_convert_encoding($tx, 'UTF-8', $sjEnc);
	return (mb_convert_encoding($txUTF, $sjEnc, 'UTF-8') === $tx);
}

function sj5c_enc($tx, $sjEnc= 'SJIS-win'){ // 16進表記に変更
	if(!sj5c_isSJIS($tx, $sjEnc)){ return $tx; }
	$c_5c = chr(0x5c);
	$rc = '';
	$ml = mb_strlen($tx, $sjEnc);
	for($i = 0; $i < $ml; $i++){
		$mc = mb_substr($tx, $i, 1, $sjEnc);
		$l = strlen($mc);
		if($l > 1 && strpos($mc, $c_5c) !== FALSE){
			$rc .= '{0x';
			for($j = 0; $j < $l; $j++){
				$c = substr($mc, $j, 1);
				$rc .= sprintf("%02x", ord($c));
			}
			$rc .= '}';
		}else{ $rc .= $mc; }
	}
	return $rc;
}
function sj5c_dec($tx){ // 16進表記から元に戻す
	$rc = '';
	$key = array(); $ptn = array(); $rep = array();
	if(preg_match_all('/\{0x([\d|a-f]{2}5c)\}/i', $tx, $m)){
		foreach($m[1] as $i=>$n){
			$kk = strtolower($n);
			if(isset($key[(string)$kk])){ continue; }
			$num = count($ptn);
			$key[(string)$kk] = $num;
			$ptn[$num] = '/\{0x' . $kk . '\}/i';
			list($r1, $r2) = sscanf($kk, "%02x%02x");
			$rep[$num] = chr($r1) . chr($r2); 
		}
	}
	if(count($ptn) < 1){ return $tx; }
	$rc = preg_replace($ptn, $rep, $tx);
	return (($rc !== NULL) ? $rc : $tx);
}

function sj5c_chmod($fn, $m){ return chmod(sj5c_enc($fn), $m); }
function sj5c_file_exists($fn){ return file_exists(sj5c_enc($fn)); }
function sj5c_file_put_contents($fn, $d){
	list($a1, $a2, $a3, $a4) = func_get_args();
	$n = func_num_args();
	if($n < 3)     { $rc = file_put_contents(sj5c_enc($fn), $d); }
	else if($n < 4){ $rc = file_put_contents(sj5c_enc($fn), $d, $a3); }
	else           { $rc = file_put_contents(sj5c_enc($fn), $d, $a3, $a4); }
	return $rc;
}
function sj5c_is_dir($fn){ return is_dir(sj5c_enc($fn)); }
function sj5c_mkdir($fn, $m=0777, $r=FALSE)
	{ return mkdir(sj5c_enc($fn), $m, $r); }
function sj5c_unlink($fn){ return unlink(sj5c_enc($fn)); }

$gWaitID = (($v = $_GET['WAITID']) != '') ? uniDecode($v) : '';
$gCB = (($v = $_POST['cb']) != '') ? $v : 'sg_tenpu_save_end';
$gPara = array(
	'dir' => uniDecode($_POST['dir']),
	'cnt' => (int)$_POST['cnt'],
	'buf' => array(),
);
for($i = 0; $i < $gPara['cnt']; $i++){
	if(($f = uniDecode($_POST["fnm${i}"])) == ''){ continue; }
	$gPara['buf'][(string)$f] = &$_POST["buf${i}"];
}

function P_main($para, &$RES){
	$RES = array(
		'rc' => 0,
		'fl' => array(),
	);
	$dnm = $para['dir'];
	$dir = "." ;
	if($dnm != ''){
		$dir .= "/$dnm";
		if(!sj5c_is_dir($dir)){
			if(!sj5c_mkdir($dir)){ return -__LINE__; }
			if(!sj5c_chmod($dir, 0777)){ return -__LINE__; }
		}
	}
	foreach($para['buf'] as $fnm=>$buf){
		if(($data = base64_decode($buf)) === FALSE){ return -__LINE__; }
		$file = "$dir/$fnm";
		if(sj5c_file_exists($file)){ sj5c_unlink($file); }
		if(($s = sj5c_file_put_contents($file, $data)) === FALSE)
			{ return -__LINE__; }
		if(!sj5c_chmod($file, 0666)){ return -__LINE__; }
		$RES['fl'][(string)mb_utf8($fnm)] =
			(($dnm != '') ? (rawurlencode(sj5c_enc($dnm)) . "/") : '') .
			rawurlencode(sj5c_enc($fnm));
		$RES['rc']++;
	}
	return $RES['rc'];
}

if(($rc = P_main($gPara, $DATA)) < 0){ $DATA['rc'] = $rc; }
ob_end_clean();
header("Content-Type: application/json; charset=UTF-8");
echo json_encode($DATA);
?>
